@@ -16,7 +16,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION $OID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 use constant INIT_FIELD_COUNTER  => -1;
 use constant NO_IMATCH           => -1; # for index()
@@ -187,7 +187,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 sub _extract_dotnet {
     my($self, @args) = @_;
@@ -248,9 +248,31 @@ sub _fix_generic {
 
 sub _parse_maxthon {
     my($self, $moz, $thing, $extra, @others) = @_;
-    my @omap = grep { $_ } map { split RE_SC_WS_MULTI, $_ } @others;
+    my $is_30 =    $extra
+                && $extra->[0]
+                && index( $extra->[0], 'AppleWebKit' ) != NO_IMATCH;
     my($maxthon, $msie, @buf);
 
+    if ( $is_30 ) {
+        # yay, new nonsense with the new version
+        my @new;
+        for my $i (0..$#others) {
+            if ( index( $others[$i], 'Maxthon') != NO_IMATCH ) {
+                @new        = split m{\s+}xms, $others[$i];
+                $maxthon    = shift @new;
+                $extra    ||= [];
+                unshift @{ $extra }, shift @new;
+                $others[$i] = '';
+                last;
+            }
+        }
+        @others = grep { $_ } @others, @new;
+        $self->_parse_safari( $moz, $thing, $extra, @others );
+        $self->[UA_NAME] = 'Maxthon';
+    }
+    else {
+    my @omap = grep { $_ } map { split RE_SC_WS_MULTI, $_ } @others;
+
     foreach my $e ( @omap, @{$thing} ) { # $extra -> junk
         if ( index(uc $e, 'MAXTHON') != NO_IMATCH ) {
             $maxthon = $e;
@@ -263,6 +285,7 @@ sub _parse_maxthon {
         }
         push @buf, $e;
     }
+    }
 
     if ( ! $maxthon ) {
         warn ERROR_MAXTHON_VERSION . "\n";
@@ -270,17 +293,24 @@ sub _parse_maxthon {
         return;
     }
 
-    if ( ! $msie ) {
-        warn ERROR_MAXTHON_MSIE . "\n";
-        $self->[UA_UNKNOWN] = 1;
-        return;
+    if ( $is_30 ) {
+        if ( $self->[UA_LANG] ) {
+            push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+            $self->[UA_LANG] = undef;
+        }
+    }
+    else {
+        if ( ! $msie ) {
+            warn ERROR_MAXTHON_MSIE . "\n";
+            $self->[UA_UNKNOWN] = 1;
+            return;
+        }
+        $self->_parse_msie(
+            $moz, [ undef, @buf ], undef, split RE_WHITESPACE, $msie
+        );
     }
 
-    $self->_parse_msie(
-        $moz, [ undef, @buf ], undef, split RE_WHITESPACE, $msie
-    );
-
-    my(undef, $mv) = split RE_WHITESPACE, $maxthon;
+    my(undef, $mv) = split $is_30 ? RE_SLASH : RE_WHITESPACE, $maxthon;
     my $v = $mv      ? $mv
           : $maxthon ? '1.0'
           :            do { warn ERROR_MAXTHON_VERSION . "\n"; 0 }
@@ -391,7 +421,7 @@ sub _parse_safari {
                             : $ipad ? 'iPad'
                             :         'Safari';
     $self->[UA_VERSION_RAW] = $vx;
-    $self->[UA_TOOLKIT]     = $extra ? [ split RE_SLASH, $extra->[0] ] : [];
+    $self->[UA_TOOLKIT]     = $extra ? [ split RE_SLASH, shift @{ $extra } ] : [];
     if ( $thing->[-1] && length($thing->[LAST_ELEMENT]) <= 5 ) {
         # todo: $self->_is_lang_field($junk)
         # in here or in _post_parse()
@@ -435,18 +465,29 @@ sub _parse_safari {
     }
 
     push @{$self->[UA_EXTRAS]}, @junk if @junk;
+    push @{$self->[UA_EXTRAS]}, @{$extra} if $extra;
 
     return 1;
 }
 
 sub _parse_chrome {
     my($self, $moz, $thing, $extra, @others) = @_;
-    my $chx                  = pop @others;
-    my($chrome, $safari)     = split RE_WHITESPACE, $chx;
-    push @others, $safari;
+    my $chx = pop @others;
+    my($chrome, $safari, @rest) = split RE_WHITESPACE, $chx;
+    my $opera;
+    if ( $rest[0] && index( $rest[0], 'OPR/', 0) != NO_IMATCH ) {
+        $opera = shift @rest;
+        if ( ref $extra eq 'ARRAY' ) {
+            unshift @{ $extra }, $chrome;
+        }
+        push @others, @rest, $safari;
+    }
+    else {
+        push @others, $safari;
+    }
     $self->_parse_safari($moz, $thing, $extra, @others);
-    my($name, $version)      = split RE_SLASH, $chrome;
-    $self->[UA_NAME]         = $name;
+    my($name, $version)      = split RE_SLASH, $opera || $chrome;
+    $self->[UA_NAME]         = $opera ? 'Opera' : $name;
     $self->[UA_VERSION_RAW]  = $version;
     return 1;
 }
@@ -885,7 +926,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 sub _is_opera_pre {
     my($self, $moz) = @_;
@@ -1031,7 +1072,7 @@ use vars qw( $VERSION );
 use Carp qw( croak );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 sub dumper {
     my($self, @args) = @_;
@@ -1129,7 +1170,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 #TODO: new accessors
 #wap
@@ -1189,7 +1230,8 @@ sub toolkit {
             );
 }
 
-package Parse::HTTP::UserAgent::Base::Accessors::toolkit;
+package
+Parse::HTTP::UserAgent::Base::Accessors::toolkit;
 use strict;
 use warnings;
 use overload '""',    => 'name',
@@ -1220,7 +1262,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION );
 
-$VERSION = '0.35';
+$VERSION = '0.39';
 
 use base qw(
     Parse::HTTP::UserAgent::Base::IS
@@ -1579,8 +1621,8 @@ generated with an automatic build tool. If you experience problems
 with this version, please install and use the supported standard
 version. This version is B<NOT SUPPORTED>.
 
-This document describes version C<0.35> of C<Parse::HTTP::UserAgent>
-released on C<14 May 2012>.
+This document describes version C<0.39> of C<Parse::HTTP::UserAgent>
+released on C<2 December 2013>.
 
 Quoting L<http://www.webaim.org/blog/user-agent-string-history/>:
 
@@ -1759,12 +1801,11 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 - 2012 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2013 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.12.3 or, 
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.16.2 or,
 at your option, any later version of Perl 5 you may have available.
-
 =cut
